#!/usr/bin/awk -f
#
# gcw-work.awk -- (C) 2005 Geoffrey Reynolds.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#
# usage: gcw-work.awk [FROM [TO [B1 [B2 [CURVES]]]]] [FILE ...]
#
# If FILE contains factors of n*a^n+c where c=1 or c=-1 from the
# [Generalized] Cullen and Woodhall project, e.g. 2+.txt, 2-.txt, 3+.txt etc.
# (see http://www.leyland.vispa.com/numth/factorization/cullen_woodall/cw.htm)
# then those in the range FROM <= n <= TO that have not been completely
# factored will be output one per line in the format
#
#  ECM2=k,b,m,c,B1,B2,CURVES
#
# where k*b^m+c = n*a^n+c and m is maximal. This is the format used by Prime95
# version 24. As of version 24.11 it is prpbebly only worthwhile for b=2 (i.e.
# for a=2,4,8), but all bases 2 to 12 are handled anyway.
#


#
# If another integer argument exists then return it, otherwise return default.
#
function get_int_arg(dflt,   tmp)
{
  while(ARGC >= ++arg_ind)
  {
    if (ARGV[arg_ind] ~ /^[0-9]+$/)
    {
      tmp = ARGV[arg_ind];
      ARGV[arg_ind] = "";
      return tmp;
    }
  }
  return dflt;
}

#
# product("2.3.3.23") returns 2*3*3*23 etc.
#
function product(str,    A, i, p)
{
  p = 1;
  split(str,A,/\./);
  for(i in A)
    p *= A[i];
  return p;
}

#
# Find a,c in n*a^n+c given g1=b+c and g2=2*b^2+c
#
function init_ac(g1, g2)
{
  c = 1;
  if(2*(g1-1)*(g1-1)+1 < g2)
    c = -1;
  a = g1-c;
}

#
# Express a as b^m so that m is maximised.
#
function init_bm()
{
  b = a;
  m = 1;
  if(a==4) {b=2; m=2}
  if(a==8) {b=2; m=3}
  if(a==9) {b=3; m=2}
}

#
# Print a worktodo.ini line for k*b^x+c = n*a^n+c with x maximised.
#
function worktodo_entry(n,    k, x)
{
  k = n;
  x = n*m;
  while(k%b==0)
  {
    x = x+1;
    k /= b;
  }
  printf("ECM2=%d,%d,%d,%d,%d,%d,%d\n",k,b,x,c,B1,B2,curves);
}


#
# Process command line. This avoids the need for the usual -vnmin=100 etc.
#
BEGIN {
  nmin = get_int_arg(100);
  nmax = get_int_arg(1000);
  B1 =   get_int_arg(1000000);
  B2 =   get_int_arg(0);
  curves = get_int_arg(100);
}

#
# Use the first two lines of input to deduce constants c,b,m
#
$1 == 1 { g1 = product($2); }

$1 == 2 { g2 = product($2); init_ac(g1,g2); init_bm(); }

#
# Main loop.
#
$NF ~ /C/ && $1 >= nmin && $1 <= nmax { worktodo_entry($1); }
